@@ -33,7 +33,7 @@ msm <- function(formula,   # formula with  observed Markov states   ~  observati
                 pci = NULL,
                 cl = 0.95, # width of confidence intervals
                 fixedpars = NULL, # specify which parameters to fix. TRUE for all parameters
-                center = TRUE, # center covariates at their means
+                center = TRUE, # center covariates at their means during optimisation
                 opt.method = c("optim","nlm"),
                 hessian = TRUE,
                 use.deriv = TRUE,
@@ -83,12 +83,12 @@ msm <- function(formula,   # formula with  observed Markov states   ~  observati
     msmdata.obs <- msm.form.data(formula, subject, obstype, obstrue, covariates, data,
                                  hcovariates, misccovariates, initcovariates,
                                  qmodel, emodel, hmodel, cmodel, dmodel, exacttimes, center)
-### EXPAND DATA AND MODEL FOR TIME DEPENDENT INTENSITIES 
-    if (!is.null(pci)) { 
+### EXPAND DATA AND MODEL FOR TIME DEPENDENT INTENSITIES
+    if (!is.null(pci)) {
         tdmodel <- msm.pci(pci, msmdata.obs, qmodel, cmodel, center)
-        if (is.null(tdmodel)) # supplied cut points not in range of data 
+        if (is.null(tdmodel)) # supplied cut points not in range of data
             pci <- NULL
-        else { 
+        else {
             cmodel <- tdmodel$cmodel
             msmdata.obs.orig <- msmdata.obs
             names(msmdata.obs.orig)[names(msmdata.obs.orig) %in% c("covmat","covmat.orig")] <- c("cov","cov.orig") # used in bootstrap
@@ -119,7 +119,7 @@ msm <- function(formula,   # formula with  observed Markov states   ~  observati
     msmdata$cov <- msmdata.obs$covmat
     msmdata$cov.orig <- msmdata.obs$covmat.orig
     msmdata$covlabels.orig <- msmdata.obs$covlabels.orig
-    
+
 ### MODEL FOR COVARIATES ON INTENSITIES
     qcmodel <-
         if (msmdata$covdata$ncovs > 0)
@@ -154,7 +154,9 @@ msm <- function(formula,   # formula with  observed Markov states   ~  observati
         class(hmodel) <- "hmodel"
     }
     if (!is.null(initcovariates)) {
-        hmodel <- msm.form.icmodel(hmodel, msmdata$icovdata, initcovinits)
+        if (hmodel$hidden)
+            hmodel <- msm.form.icmodel(hmodel, msmdata$icovdata, initcovinits)
+        else warning("initprobs and initcovariates ignored for non-hidden Markov models")
     }
     else if (hmodel$hidden) {
         hmodel <- c(hmodel, list(nicovs=rep(0, hmodel$nstates-1), nicoveffs=0))
@@ -172,7 +174,6 @@ msm <- function(formula,   # formula with  observed Markov states   ~  observati
     if (p$fixed) {
         p$lik <- lik.msm(p$inits, msmdata, qmodel, qcmodel, cmodel, hmodel, p)
         p$deriv <- if (!hmodel$hidden && cmodel$ncens==0) deriv.msm(p$inits, msmdata, qmodel, qcmodel, cmodel, hmodel, p) else NULL
-        p$allinits[p$hmmpars] <- msm.mninvlogit.transform(p$allinits[p$hmmpars], hmodel$plabs, hmodel$parstate)
         p$params.uniq <- p$allinits[!duplicated(abs(p$constr))]
         p$params <- p$allinits[!duplicated(abs(p$constr))][abs(p$constr)]*sign(p$constr)
         p$foundse <- FALSE
@@ -211,7 +212,11 @@ msm <- function(formula,   # formula with  observed Markov states   ~  observati
         p$params[p$hmmpars] <- msm.mninvlogit.transform(p$params[p$hmmpars], hmodel$plabs, hmodel$parstate)
         p$params.uniq <- p$params[!duplicated(abs(p$constr))]
         p$params <- p$params[!duplicated(abs(p$constr))][abs(p$constr)]*sign(p$constr)
-        if (hessian && all(eigen(opt$hessian)$values > 0)) {
+
+        if (hessian &&
+            all(!is.na(opt$hessian)) && all(!is.nan(opt$hessian)) && all(is.finite(opt$hessian)) &&
+            all(eigen(opt$hessian)$values > 0))
+        {
             p$foundse <- TRUE
             p$covmat <- matrix(0, nrow=p$npars, ncol=p$npars)
             p$covmat[p$optpars,p$optpars] <- solve(0.5 * opt$hessian)
@@ -301,7 +306,7 @@ msm <- function(formula,   # formula with  observed Markov states   ~  observati
                        )
     attr(msmobject, "fixed") <- p$fixed
     class(msmobject) <- "msm"
-    q <- qmatrix.msm(msmobject) # intensity matrix with centered covariates
+    q <- qmatrix.msm(msmobject, covariates=(if(center) "mean" else 0)) # intensity matrix on natural scale
     msmobject$Qmatrices$baseline <- q$estimates
     msmobject$QmatricesSE$baseline <- q$SE
     msmobject$QmatricesL$baseline <- q$L
@@ -311,14 +316,14 @@ msm <- function(formula,   # formula with  observed Markov states   ~  observati
         msmobject$EmatricesSE <- EmatricesSE
         msmobject$EmatricesL <- EmatricesL
         msmobject$EmatricesU <- EmatricesU
-        e <- ematrix.msm(msmobject) # misc matrix with centered covariates
+        e <- ematrix.msm(msmobject, covariates=(if(center) "mean" else 0)) # misc matrix on natural scale
         msmobject$Ematrices$baseline <- e$estimates
         msmobject$EmatricesSE$baseline <- e$SE
         msmobject$EmatricesL$baseline <- e$L
         msmobject$EmatricesU$baseline <- e$U
     }
-    ## Calculate mean sojourn times with centered covariates
-    msmobject$sojourn <- sojourn.msm(msmobject)
+    ## Calculate mean sojourn times at baseline
+    msmobject$sojourn <- sojourn.msm(msmobject, covariates=(if(center) "mean" else 0))
     msmobject
 }
 
@@ -350,7 +355,6 @@ msm.fixdiag.ematrix <- function(ematrix)
 
 msm.form.qmodel <- function(qmatrix, qconstraint=NULL, exacttimes=FALSE, gen.inits=FALSE, formula, subject, data, censor, censor.states, analyticp)
 {
-### INTENSITY MATRIX (INPUT: qmatrix, qconstraint; OUTPUT: nstates, nintens, qmatrix, qvector, baseconstr, nintenseffs)
     if (gen.inits)
         qmatrix <- crudeinits.msm(formula, subject, qmatrix, data, censor, censor.states)
     msm.check.qmatrix(qmatrix)
@@ -465,11 +469,11 @@ msm.form.data <- function(formula, subject=NULL, obstype=NULL, obstrue=NULL, cov
     statetimerows.kept <- (1:n)[! ((1:n) %in% droprows)]
     if (is.null(subject)) subject <- rep(1, nrow(mf))
     obstype <- msm.form.obstype(obstype, n, state, statetimerows.kept, dmodel, exacttimes)
-    obstrue <- msm.form.obstrue(obstrue, nrow(mf), hmodel)   
+    obstrue <- msm.form.obstrue(obstrue, nrow(mf), hmodel)
     subjrows.kept <- (1:n) [!is.na(subject)]
     otrows.kept <- statetimerows.kept[!is.na(obstype)]
     omrows.kept <- statetimerows.kept[!is.na(obstrue)]
-    
+
     ## Don't drop NA covariates on the transition process at the subject's final observation, since they are not used
     lastobs <- c(which(subject[1:(n-1)] != subject[2:n]), n)
     ##Parse covariates formula and extract data
@@ -522,8 +526,8 @@ msm.form.data <- function(formula, subject=NULL, obstype=NULL, obstrue=NULL, cov
         final.rows <- intersect(final.rows, icovdata$covrows.kept)
     subject <- subject[final.rows]
     time <- subset(time, statetimerows.kept %in% final.rows)
-    msm.check.times(time, subject)
     state <- subset(state, statetimerows.kept %in% final.rows)
+    msm.check.times(time, subject, state)
     obstype <- subset(obstype, statetimerows.kept %in% final.rows)
     obstrue <- subset(obstrue, statetimerows.kept %in% final.rows)
     covmat <- covmat.orig <- numeric()
@@ -585,7 +589,7 @@ msm.check.state <- function(nstates, state=NULL, censor)
     invisible()
 }
 
-msm.check.times <- function(time, subject)
+msm.check.times <- function(time, subject, state=NULL)
 {
 ### Check if any individuals have only one observation (after excluding missing data)
     subj.num <- match(subject,unique(subject)) # avoid problems with factor subjects with empty levels
@@ -616,6 +620,16 @@ msm.check.times <- function(time, subject)
         plural <- if (length(badsubjs)==1) "" else "s"
         stop ("Observations within subject", plural, " ", badlist, " are not ordered by time")
     }
+### Check if any consecutive observations are made at the same time, but with different states
+    if (!is.null(state)){
+        prevsubj <- c(-Inf, subj.num[1:length(subj.num)-1])
+        prevtime <- c(-Inf, time[1:length(time)-1])
+        prevstate <- c(-Inf, state[1:length(state)-1])
+        sametime <- (subj.num==prevsubj & prevtime==time & prevstate!=state)
+        badlist <- paste(paste(which(sametime)-1, which(sametime), sep=" and "), collapse=", ")
+        if (any(sametime))
+            warning("Different states observed at the same time on the same subject at observations ", badlist)
+    }
     invisible()
 }
 
@@ -692,6 +706,7 @@ msm.impute.censored <- function(fromstate, tostate, Pmat, cmodel)
 msm.check.model <- function(fromstate, tostate, obs, subject, obstype=NULL, qmatrix, cmodel)
 {
     n <- length(fromstate)
+    qmatrix <- qmatrix / mean(qmatrix[qmatrix>0]) # rescale to avoid false warnings with small rates
     Pmat <- MatrixExp(qmatrix)
     Pmat[Pmat < 1e-16] <- 0
     imputed <- msm.impute.censored(fromstate, tostate, Pmat, cmodel)
@@ -701,14 +716,14 @@ msm.check.model <- function(fromstate, tostate, obs, subject, obstype=NULL, qmat
 
     if (identical(all.equal(min(unitprob, na.rm=TRUE), 0),  TRUE))
     {
-        badobs <- min (obs[unitprob==0], na.rm = TRUE)
-        warning ("Data inconsistent with transition matrix for model without misclassification:\n",
+        badobs <- which.min(unitprob)
+        warning ("Data may be inconsistent with transition matrix for model without misclassification:\n",
                  "individual ", if(is.null(subject)) "" else subject[obs==badobs], " moves from state ", fromstate[obs==badobs],
                  " to state ", tostate[obs==badobs], " at observation ", badobs, "\n")
     }
     if (any(qunit[obstype==2]==0)) {
         badobs <- min (obs[qunit==0 & obstype==2], na.rm = TRUE)
-        warning ("Data inconsistent with intensity matrix for observations with exact transition times and no misclassification:\n",
+        warning ("Data may be inconsistent with intensity matrix for observations with exact transition times and no misclassification:\n",
                  "individual ", if(is.null(subject)) "" else subject[obs==badobs], " moves from state ", fromstate[obs==badobs],
                  " to state ", tostate[obs==badobs], " at observation ", badobs)
     }
@@ -1143,7 +1158,7 @@ likderiv.msm <- function(params, deriv=0, msmdata, qmodel, qcmodel, cmodel, hmod
     hmodel$links <- hmodel$links - 1
     pars[plabs == "p"] <- exp(pars[plabs == "p"])
     initprobs <- c(1 - sum(pars[plabs=="initp"]), pars[plabs %in% c("initp","initp0")])
-    initprobs <- initprobs / initprobs[1] ## initprobs[1] documented as not allowed to be zero. 
+    initprobs <- initprobs / initprobs[1] ## initprobs[1] documented as not allowed to be zero.
 
     if (!do.what %in% c(0,1,3)) stop("deriv should be 0, 1 or 3")
     lik <- .C("msmCEntry",
@@ -1163,7 +1178,7 @@ likderiv.msm <- function(params, deriv=0, msmdata, qmodel, qcmodel, cmodel, hmod
               as.integer(msmdata$covdata$whichcov), # this is really part of the model
               as.integer(msmdata$nocc),
               as.integer(msmdata$whicha),
-              as.integer(if(do.what==3) msmdata$obstype.obs else msmdata$obstype), # need per-observation obstype when doing derivs by subject. 
+              as.integer(if(do.what==3) msmdata$obstype.obs else msmdata$obstype), # need per-observation obstype when doing derivs by subject.
 
               ## data for HMM or censored
               as.integer(match(msmdata$subject, unique(msmdata$subject))),
@@ -1195,7 +1210,7 @@ likderiv.msm <- function(params, deriv=0, msmdata, qmodel, qcmodel, cmodel, hmod
               as.integer(qmodel$npars),
               as.integer(qmodel$ndpars),
               as.integer(qcmodel$ndpars),
-              as.integer(msmdata$nobs), # number of aggregated observations 
+              as.integer(msmdata$nobs), # number of aggregated observations
               as.integer(msmdata$n), # number of observations
               as.integer(msmdata$npts),  # HMM only
               as.integer(rep(qcmodel$ncovs, qmodel$npars)),
@@ -1225,7 +1240,7 @@ likderiv.msm <- function(params, deriv=0, msmdata, qmodel, qcmodel, cmodel, hmod
             else lik$returned[1:qmodel$ndpars]*exp(params[1:qmodel$ndpars])
         lik$returned <- lik$returned[setdiff(seq(along=lik$returned), p$constr[p$fixedpars])]
     }
-    
+
     ## subject-specific derivatives, to use for score residuals
     else if (deriv==3) {
         lik$returned <- matrix(lik$returned, nrow=msmdata$npts)
@@ -1247,7 +1262,7 @@ deriv.msm <- function(params, ...)
     likderiv.msm(params, deriv=1, ...)
 }
 
-## Convert vector of MLEs into matrices 
+## Convert vector of MLEs into matrices
 
 msm.form.output <- function(whichp, model, cmodel, p)
 {
@@ -1291,7 +1306,7 @@ msm.form.output <- function(whichp, model, cmodel, p)
          )
 }
 
-## Format hidden Markov model estimates and CIs 
+## Format hidden Markov model estimates and CIs
 
 msm.form.houtput <- function(hmodel, p)
 {
@@ -1398,7 +1413,7 @@ crudeinits.msm <- function(formula, subject, qmatrix, data=NULL, censor=NULL, ce
     estmat
 }
 
-### Construct a model with time-dependent transition intensities. 
+### Construct a model with time-dependent transition intensities.
 ### Form a new dataset with censored states and extra covariate, and
 ### form a new censor model, given change times in tcut
 
@@ -1422,13 +1437,13 @@ msm.pci <- function(tcut, dat, qmodel, cmodel, center)
     if (dat$ncovs > 0){
         extra$covmat <- as.data.frame(matrix(NA, nrow=nextra, ncol=ncol(old$covmat)))
         extra$covmat.orig <- as.data.frame(matrix(NA, nrow=nextra, ncol=ncol(old$covmat.orig)))
-        rownames(old$covmat.orig) <- rownames(old$covmat) <- 1:nrow(old) 
+        rownames(old$covmat.orig) <- rownames(old$covmat) <- 1:nrow(old)
         rownames(extra$covmat.orig) <- rownames(extra$covmat) <- nrow(old) + 1:nrow(extra) # get rid of dup / null rownames errors
     }
-    ## merge new and old observations 
+    ## merge new and old observations
     new <- rbind(old, extra)
     new <- new[order(new$subject, new$time),]
-    label <- if (cmodel$ncens > 0) max(cmodel$censor)*2 else qmodel$nstates + 1 
+    label <- if (cmodel$ncens > 0) max(cmodel$censor)*2 else qmodel$nstates + 1
     new$state[is.na(new$state)] <- label
     ## Only keep cutpoints within range of each patient's followup
     mintime <- tapply(old$time, old$subject, min)[as.character(unique(old$subject))]
@@ -1437,14 +1452,14 @@ msm.pci <- function(tcut, dat, qmodel, cmodel, center)
     new <- new[new$time >= rep(mintime, nobspt) & new$time <= rep(maxtime, nobspt), ]
 
     ## drop imputed observations at times when there was already an observation
-    ## assumes there wasn't already duplicated obs times 
+    ## assumes there wasn't already duplicated obs times
     prevsubj <- c(NA,new$subject[1:(nrow(new)-1)]); nextsubj <- c(new$subject[2:nrow(new)], NA)
     prevtime <- c(NA,new$time[1:(nrow(new)-1)]); nexttime <- c(new$time[2:nrow(new)], NA)
     prevstate <- c(NA,new$state[1:(nrow(new)-1)]); nextstate <- c(new$state[2:nrow(new)], NA)
     new <- new[!((new$subject==prevsubj & new$time==prevtime & new$state==label & prevstate!=label) |
                  (new$subject==nextsubj & new$time==nexttime & new$state==label & nextstate!=label))
                ,]
-    
+
     ## Carry last value forward for other covariates
     if (dat$ncovs > 0) {
         eind <- which(is.na(new$covmat[,1]))
@@ -1454,7 +1469,7 @@ msm.pci <- function(tcut, dat, qmodel, cmodel, center)
             eind <- which(is.na(new$covmat[,1]))
         }
     }
-    
+
     ## constants in dataset
     new <- as.list(new)
     new$nobs <- new$n <- n <- length(new$state)
@@ -1473,7 +1488,7 @@ msm.pci <- function(tcut, dat, qmodel, cmodel, center)
                 " greater than or equal to maximum observed time of ",max(dat$time))
     tcut <- tcut[tcut > min(dat$time) & tcut < max(dat$time)]
     ntcut <- length(tcut)
-    if (ntcut==0) 
+    if (ntcut==0)
         res <- NULL # no cut points in range of data, continue with no time-dependent model
     else {
         ## Insert new covariate in data representing time period
@@ -1482,14 +1497,14 @@ msm.pci <- function(tcut, dat, qmodel, cmodel, center)
             tcovlabel <- paste(tcovlabel, ".1", sep="")
         tcov <- factor(cut(new$time, c(-Inf,tcut,Inf), right=FALSE))
         levs <- levels(tcov)
-        levels(tcov) <- gsub(" ","", levs) # get rid of spaces in e.g. [10, Inf) levels 
+        levels(tcov) <- gsub(" ","", levs) # get rid of spaces in e.g. [10, Inf) levels
         assign(tcovlabel, tcov)
         mm <- model.matrix(as.formula(paste("~", tcovlabel)))[,-1,drop=FALSE]
         new$covmat <- cbind(new$covmat, mm)
         new$covmat.orig <- if(is.null(new$covmat.orig)) data.frame(timeperiod=tcov) else cbind(new$covmat.orig, timeperiod=tcov)
         if (center) new$covmat <- sweep(new$covmat, 2, colMeans(new$covmat))
-        
-        ## new censoring model 
+
+        ## new censoring model
         cmodel$ncens <- cmodel$ncens + 1
         cmodel$censor <- c(cmodel$censor, label)
         cmodel$states <- c(cmodel$states, 1:qmodel$nstates)
